#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" ${1+"$@"}

set OptionDef {
    -?, --help | EmitHelp | This help information
    -r, --raw | $camera _setAttribute raw_format %1$s | Add raw bayer data to jpeg metadata (see below)
    -t, --timeout | after %1$s | Time (in ms) before takes picture and shuts down (if not specified, set to 5s)
    -x, --exif | $camera _setAttribute exif_tags %1$s | EXIF tag to apply to captures (format as 'key value' list)
    -md, --mode | $camera _setAttribute sensor_mode %1$s | Force sensor mode. 0=auto. See docs for other modes available
    -sh, --sharpness | $camera _setAttribute sharpness %1$s | Set image sharpness (-100 to 100)
    -co, --contrast | $camera _setAttribute contrast %1$s | Set image contrast (-100 to 100)
    -br, --brightness | $camera _setAttribute brightness %1$s | Set image brightness (0 to 100)
    -sa, --saturation | $camera _setAttribute saturation %1$s | Set image saturation (-100 to 100)
    -ISO, --ISO | $camera _setAttribute iso %1$s | Set capture ISO
    -ev, --ev | $camera _setAttribute exposure_compensation %1$s | Set EV compensation - steps of 1/6 stop
    -ex, --exposure | $camera _setAttribute exposure_mode %1$s | Set exposure mode (see below)
    -awb, --awb | $camera _setAttribute awb_mode %1$s | Set AWB mode (see below)
    -ifx, --imxfx | $camera _setAttribute image_effect %1$s  | Set image effect (see below)
    -cfx, --colfx | $camera _setAttribute color_effects %1$s | Set colour effect (U:V)
    -mm, --metering | $camera _setAttribute meter_mode %1$s | Set metering mode (see below)
    -rot, --rotation | $camera _setAttribute rotation %1$s | Set image rotation (0-359)
    -hf, --hflip | $camera _setAttribute hflip  [expr {%1$s?1:0}] | Set horizontal flip (boolean)
    -vf, --vflip | $camera _setAttribute vflip [expr {%1$s?1:0}] | Set vertical flip (boolean)
    -roi, --roi | $camera _setAttribute crop %1$s | Set region of interest (x,y,w,d as normalised coordinates [0.0-1.0])
    -ss, --shutter | $camera _setAttribute shutter_speed %1$s | Set shutter speed in microseconds
    -awbg, --awbgains | $camera _setAttribute awb_gains %1$s | Set AWB gains - AWB mode must be off
    -drc, --drc | $camera _setAttribute drc_strength [%1$s | Set DRC Level (see below)
    -st, --stats | $camera _setAttribute still_stats [expr {%1$s?1:0}] | Force recomputation of statistics on stills capture pass (boolean)
    -dg, --digitalgain | $camera _setAttribute digital_gain %1$s | Set the digtial gain (floating point)
}

set HelpFmt {

Runs camera for specific time, and take JPG capture at end if requested

usage: pistill.tcl [options] outputFileName

Image parameter commands:
%1$s

Exposure mode options :
    off, auto, night, nightpreview, backlight, spotlight, sports, snow, beach,
    verylong, fixedfps, antishake or fireworks

Flicker avoid mode options :
    off, auto, 50hz or 60hz

AWB mode options :
    off, auto, sun, cloud, shade, tungsten, fluorescent, incandescent, flash or
    horizon

Image Effect mode options :
    none, negative, solarise, sketch, denoise, emboss, oilpaint, hatch, gpen,
    pastel, watercolour, film, blur, saturation, colourswap, washedout,
    posterise, colourpoint, colourbalance or cartoon

Metering Mode options :
    average, spot, backlit or matrix

Dynamic Range Compression (DRC) options :
    off, low, med or high

}

##
## Emit help message and exit
##
proc EmitHelp {} {
    global HelpFmt
    global OptionDef

    set paramSection {}
    foreach paramDef [split $OptionDef "\n"] {
        set paramList [split $paramDef "|"]
        if {[llength $paramList] ne 3} {
            continue
        }
        lassign $paramList options command description
        append paramSection [format {%25.25s} [string trim $options]] "\t: " $description "\n"
    }

    puts stderr [format $HelpFmt $paramSection]
    exit 1
}

proc ProcessOptions {argList} {
    global OptionDef
    global SymbolicDict
    global camera

    set optionDict {}

    foreach paramDef [split $OptionDef "\n"] {
        set paramList [split $paramDef "|"]
        if {[llength $paramList] ne 3} {
            continue
        }
        lassign $paramList options command description
        set command [string trim $command]
        foreach option [split $options ","] {
            dict set optionDict [string trim $option] $command
        }
    }
    foreach {option value} $argList {
        if {![dict exists $optionDict $option]} {
            puts stderr "unkown option: $option"
            EmitHelp
        }
        eval [format [dict get $optionDict $option] $value]
    }
}

package require pyman
pyman import -namespace ::picamera picamera

set camera [::picamera::PiCamera new]

##
## Get the numerics for symbolics
##
foreach symbolSet {AWB_MODES CLOCK_MODES DRC_STRENGTHS EXPOSURE_MODES FLASH_MODES IMAGE_EFFECTS METER_MODES STEREO_MODES} {
    set SymbolicDict($symbolSet) [$camera _getAttribute $symbolSet]
}

switch -exact -- $argc {
    0 {
        EmitHelp
    }
    1 {
        set ofn [lindex $argv 0]
    }
    default {
        set ofn [lindex $argv end]
        ProcessOptions [lrange $argv 0 end-1]
    }
}

if {$ofn in {"-?" "--help"}} {
    EmitHelp
}

$camera capture $ofn
$camera close
